@@ -1,12 +1,14 @@
 /*
  * echoping : uses the TCP echo service to measure (roughly) response times.
  * 
- * Written by Stephane Bortzmeyer <bortzmeyer@pasteur.fr>. A lot of code stolen
- * from Richard Stevens' book "Unix network programming" and Pierre Beyssac's
- * "bing" tool. 
+ * Written by Stephane Bortzmeyer <bortz@users.sourceforge.net>. See
+ * the AUTHORS file for other contributors.
+ * 
+ * $Id: echoping.c,v 4.9 2001/03/06 16:46:00 bortz Exp $
  * 
  */
 
+
 char *progname;
 unsigned short timeout_flag;
 
@@ -28,15 +30,15 @@ unsigned int successes, attempts = 0;
 unsigned int size = DEFLINE;
 unsigned int j = 0;
 struct result
-  {
-    unsigned short valid;
-    struct timeval timevalue;
-  };
+{
+  unsigned short valid;
+  struct timeval timevalue;
+};
 struct result results[MAXNUMBER];
 struct timeval good_results[MAXNUMBER];
 extern int tvcmp ();
 
-void
+int
 main (argc, argv)
      int argc;
      char *argv[];
@@ -47,16 +49,23 @@ main (argc, argv)
   signed char ch;
 
   int sockfd;
+  FILE *files = NULL;
+  CHANNEL channel;
   struct hostent *hostptr;
   struct sockaddr_in serv_addr;
   struct sockaddr_in udp_cli_addr;	/* client's Internet socket
 					 * addr */
-  struct servent *sp;
+  struct servent *sp = NULL;
   int verbose = FALSE;
   char *server_address;
   u_int addr;
   struct in_addr *ptr;
-  int n, nr;
+  int n, nr = 0;
+  int rc;
+#ifdef OPENSSL
+  int sslcode;
+  char rand_file[MAXLINE];
+#endif
   char *sendline, recvline[MAXLINE + 1];
 #ifdef ICP
   char retcode[DEFLINE];
@@ -65,7 +74,11 @@ main (argc, argv)
   struct timeval newtv, oldtv;
   void printstats ();
 
+#ifdef HAVE_USLEEP
+  double wait = 1.0;
+#else
   unsigned int wait = 1;
+#endif
   unsigned char fill = ' ';
   unsigned short fill_requested = 0;
   unsigned int i = 0;
@@ -85,17 +98,34 @@ main (argc, argv)
   char *port_name = ECHO_TCP_PORT;
   unsigned short port_to_use = USE_ECHO;
   unsigned short http = 0;
+  unsigned short smtp = 0;
   unsigned short udp = 0;
   unsigned short icp = 0;
+
+  unsigned short nocache = 0;
+
 #ifdef ICP
   icp_opcode opcode = ICP_OP_QUERY;
 #endif
 
   unsigned short ttcp = 0;
   unsigned short tcp = 0;
+  unsigned short ssl = 0;
 
   unsigned short stop_at_newlines = 1;
 
+#ifdef OPENSSL
+  SSL_METHOD *meth;
+  SSL_CTX *ctx = NULL;
+  SSL *sslh = NULL;
+#endif
+
+  int priority;
+  int priority_requested = 0;
+  int tos;
+  int tos_requested = 0;
+  char *arg_end;
+
   null_timeval.tv_sec = 0;
   null_timeval.tv_usec = 0;
   max_timeval.tv_sec = 1000000000;
@@ -113,7 +143,7 @@ main (argc, argv)
       results[i].valid = 0;
     }
   progname = argv[0];
-  while ((ch = getopt (argc, argv, "vs:n:w:dch:i:rut:f:")) != EOF)
+  while ((ch = getopt (argc, argv, "vs:n:w:dch:i:rut:f:SCp:P:aA")) != EOF)
     {
       switch (ch)
 	{
@@ -126,6 +156,9 @@ main (argc, argv)
 	case 'u':
 	  udp = 1;
 	  break;
+	case 'C':
+	  ssl = 1;
+	  break;
 	case 'd':
 	  port_name = DISCARD_TCP_PORT;
 	  port_to_use = USE_DISCARD;
@@ -136,22 +169,61 @@ main (argc, argv)
 	  stop_at_newlines = 0;
 	  break;
 	case 'i':
-	  port_name = ICP_UDP_PORT;
+	  port_name = DEFAULT_ICP_UDP_PORT;
 	  port_to_use = USE_ICP;
 	  udp = 1;
 	  icp = 1;
 	  url = optarg;
 	  break;
 	case 'h':
-	  port_name = HTTP_TCP_PORT;
+	  port_name = DEFAULT_HTTP_TCP_PORT;
 	  port_to_use = USE_HTTP;
 	  http = 1;
 	  url = optarg;
 	  break;
+	case 'a':
+	  nocache = 1;
+	  break;
+	case 'A':
+	  nocache = 2;
+	  break;
 	case 'f':
 	  fill = *optarg;
 	  fill_requested = 1;
 	  break;
+	case 'S':
+	  port_name = "smtp";
+	  port_to_use = USE_SMTP;
+	  smtp = 1;
+	  break;
+	case 'p':
+	  priority = (int) strtol (optarg, &arg_end, 0);
+	  if (arg_end == optarg || arg_end == '\0')
+	    {
+	      (void) fprintf (stderr,
+			      "%s: socket priority (-p) should be numeric.\n",
+			      progname);
+	      exit (1);
+	    }
+	  else
+	    {
+	      priority_requested = 1;
+	    }
+	  break;
+	case 'P':
+	  tos = (int) strtol (optarg, &arg_end, 0);
+	  if (arg_end == optarg || arg_end == '\0')
+	    {
+	      (void) fprintf (stderr,
+			      "%s: IP type of service (-P) should be "
+			      "numeric.\n", progname);
+	      exit (1);
+	    }
+	  else
+	    {
+	      tos_requested = 1;
+	    }
+	  break;
 	case 's':
 	  size = atoi (optarg);
 	  if (size > MAXLINE)
@@ -163,8 +235,7 @@ main (argc, argv)
 	    }
 	  if (size <= 0)
 	    {
-	      (void) fprintf (stderr,
-			      "%s: illegal packet size.\n", progname);
+	      (void) fprintf (stderr, "%s: illegal packet size.\n", progname);
 	      exit (1);
 	    }
 	  size_requested = 1;
@@ -174,8 +245,7 @@ main (argc, argv)
 	  timeout_requested = 1;
 	  if (size <= 0)
 	    {
-	      (void) fprintf (stderr,
-			      "%s: illegal timeout.\n", progname);
+	      (void) fprintf (stderr, "%s: illegal timeout.\n", progname);
 	      exit (1);
 	    }
 	  break;
@@ -184,19 +254,24 @@ main (argc, argv)
 	  if (number > MAXNUMBER)
 	    {
 	      (void) fprintf (stderr,
-			 "%s: number of iterations too large, max is %d.\n",
+			      "%s: number of iterations too large, max is %d.\n",
 			      progname, MAXNUMBER);
 	      exit (1);
 	    }
 	  if (number <= 0)
 	    {
 	      (void) fprintf (stderr,
-			   "%s: illegal number of iterations.\n", progname);
+			      "%s: illegal number of iterations.\n",
+			      progname);
 	      exit (1);
 	    }
 	  break;
 	case 'w':
+#ifdef HAVE_USLEEP
+	  wait = atof (optarg);
+#else
 	  wait = atoi (optarg);
+#endif
 	  if (wait <= 0)
 	    /* atoi returns zero when there is an error. So we cannot use 
 	       '-w 0' to specify no waiting. */
@@ -210,10 +285,12 @@ main (argc, argv)
 	  usage ();
 	}
     }
-  if (udp && ((port_to_use == USE_CHARGEN) || (port_to_use == USE_HTTP)))
+  if (udp && ((port_to_use == USE_CHARGEN) ||
+	      (port_to_use == USE_HTTP) || (port_to_use == USE_SMTP)))
     {
       (void) fprintf (stderr,
-	     "%s: I don't know how to use this port with UDP.\n", progname);
+		      "%s: I don't know how to use this port with UDP.\n",
+		      progname);
       exit (1);
     }
 /*  
@@ -226,13 +303,14 @@ main (argc, argv)
    exit (1);
    }
  */
-  if (http && (fill_requested))
+  if ((http || smtp) && (fill_requested))
     {
       (void) fprintf (stderr,
-	     "%s: Filling incompatible with HTTP connections.\n", progname);
+		      "%s: Filling incompatible with HTTP connections.\n",
+		      progname);
       exit (1);
     }
-#ifndef TTCP
+#ifndef USE_TTCP
   if (ttcp)
     {
       (void) fprintf (stderr,
@@ -240,6 +318,14 @@ main (argc, argv)
       exit (1);
     }
 #endif
+#ifndef OPENSSL
+  if (ssl)
+    {
+      (void) fprintf (stderr,
+		      "%s: not compiled with SSL support.\n", progname);
+      exit (1);
+    }
+#endif
 #ifndef HTTP
   if (http)
     {
@@ -248,6 +334,14 @@ main (argc, argv)
       exit (1);
     }
 #endif
+#ifndef SMTP
+  if (smtp)
+    {
+      (void) fprintf (stderr,
+		      "%s: Not compiled with SMTP support.\n", progname);
+      exit (1);
+    }
+#endif
 #ifndef ICP
   if (icp)
     {
@@ -256,10 +350,18 @@ main (argc, argv)
       exit (1);
     }
 #endif
-  if (http && size_requested)
+  if ((http || smtp) && size_requested)
     {
       (void) fprintf (stderr,
-		      "%s: HTTP and message size specification are incompatible.\n", progname);
+		      "%s: %s and message size specification are incompatible.\n",
+		      http ? "HTTP" : "SMTP", progname);
+      exit (1);
+    }
+  if (ssl && !http)
+    {
+      (void) fprintf (stderr,
+		      "%s: SSL is only supported for HTTP requests.\n",
+		      progname);
       exit (1);
     }
   if (udp && ttcp)
@@ -268,6 +370,28 @@ main (argc, argv)
 		      "%s: UDP and T/TCP are incompatible.\n", progname);
       exit (1);
     }
+  if (ssl && http)
+    {
+      port_name = DEFAULT_HTTPS_TCP_PORT;
+    }
+#ifndef USE_TOS
+  if (tos_requested)
+    {
+      (void) fprintf (stderr,
+		      "%s: Not compiled with Type Of Service support.\n",
+		      progname);
+      exit (1);
+    }
+#endif
+#ifndef USE_PRIORITY
+  if (priority_requested)
+    {
+      (void) fprintf (stderr,
+		      "%s: Not compiled with socket priority support.\n",
+		      progname);
+      exit (1);
+    }
+#endif
   if (!udp && !ttcp)
     {
       tcp = 1;
@@ -310,8 +434,8 @@ main (argc, argv)
 	  err_quit ("gethostbyname error for host: %s %s",
 		    server, sys_err_str ());
 	}
-      server_address = *(hostptr->h_addr_list);		/* First item of the
-							 * list */
+      server_address = *(hostptr->h_addr_list);	/* First item of the
+						 * list */
       /*
        * addr = (u_long) *server_address; 
        */
@@ -326,7 +450,7 @@ main (argc, argv)
       ptr = (struct in_addr *) malloc (sizeof (struct in_addr));
       ptr->s_addr = addr;
     }
-  if (!http)			/* Already find */
+  if (!http && !icp)		/* Already find */
     {
       if (!udp)
 	{
@@ -335,14 +459,13 @@ main (argc, argv)
 	      err_quit ("tcp_open: unknown service: %s/tcp", port_name);
 	    }
 	}
-      else if (!icp)
+      else
 	{
 	  if ((sp = getservbyname (port_name, "udp")) == NULL)
 	    {
 	      err_quit ("tcp_open: unknown service: %s/udp", port_name);
 	    }
 	}
-      /* Else ICP... */
     }
   /*
    * Fill in the structure "serv_addr" with the address of the server
@@ -364,7 +487,17 @@ main (argc, argv)
 #ifdef HTTP
   if (http)
     {
-      sendline = make_http_sendline (url, server, (int) ntohs (port));
+      sendline =
+	make_http_sendline (url, server, (int) ntohs (port), nocache);
+    }
+  else
+#endif
+#ifdef SMTP
+  if (smtp)
+    {
+      sendline = "QUIT\r\n";	/* Surprises some SMTP servers which log
+				   a frightening NOQUEUE. Anyone knows
+				   better? */
     }
   else
 #endif
@@ -385,12 +518,38 @@ main (argc, argv)
       for (i = 0; i < size; i++)
 	sendline[i] = fill;
     }
-  n = strlen (sendline) + 1;
+  n = strlen (sendline);
+
+#ifdef OPENSSL
+  if (ssl)
+    {
+      SSL_load_error_strings ();
+      SSLeay_add_ssl_algorithms ();
+      /* The following RAND_ calls are only for systems insecure
+         enough to fail to have /dev/urandom. Bug #132001 */
+      RAND_file_name (rand_file, sizeof (rand_file));
+      RAND_write_file (rand_file);
+      RAND_load_file (rand_file, 1024);
+      meth = SSLv2_client_method ();
+      if ((ctx = SSL_CTX_new (meth)) == NULL)
+	err_sys ("Cannot create a new SSL context");
+    }
+#endif
 
   for (i = 1; i <= number; i++)
     {
 
       attempts++;
+#ifdef OPENSSL
+      if (ssl)
+	/* Despite what the OpenSSL documentation says, we must
+	   allocate a new SSL structure at each iteration, otherwise,
+	   *some* SSL servers fail at the second iteration with:
+	   error:1406D0D9:SSL routines:GET_SERVER_HELLO:reuse cert type not zero 
+	   Bug #130151 */
+	if ((sslh = SSL_new (ctx)) == NULL)
+	  err_sys ("Cannot initialize SSL context");
+#endif
       if (!udp)
 	{
 	  /*
@@ -415,18 +574,53 @@ main (argc, argv)
 	      err_sys ("bind error");
 	    }
 	}
+#ifdef USE_PRIORITY
+      if (priority_requested)
+	{
+	  if (verbose)
+	    {
+	      printf ("Setting socket priority to %d (0x%02x)\n",
+		      priority, (unsigned int) priority);
+	    }
+	  if (setsockopt (sockfd,
+			  SOL_SOCKET,
+			  SO_PRIORITY,
+			  (void *) &priority, (socklen_t) sizeof (priority)))
+	    {
+	      err_sys ("Failed setting socket priority");
+	    }
+	}
+#endif
+#if USE_TOS
+      if (tos_requested)
+	{
+	  if (verbose)
+	    {
+	      printf ("Setting IP type of service octet to %d (0x%02x)\n",
+		      tos, (unsigned int) tos);
+	    }
+	  if (setsockopt (sockfd,
+			  SOL_IP,
+			  IP_TOS, (void *) &tos, (socklen_t) sizeof (tos)))
+	    {
+	      err_sys ("Failed setting IP type of service octet");
+	    }
+	}
+#endif
       if (verbose)
 	{
 	  if (tcp)
 	    {
-	      printf ("Trying to connect to internet address %s %s to transmit %u bytes...\n",
-		      inet_ntoa (*ptr), (port == 0 ? "" : text_port), n);
+	      printf
+		("Trying to connect to internet address %s %s to transmit %u bytes...\n",
+		 inet_ntoa (*ptr), (port == 0 ? "" : text_port), n);
 	    }
 #ifdef ICP
 	  if (icp)
 	    {
-	      printf ("Trying to send an ICP packet of %u bytes to the internet address %s...\n",
-		      length, inet_ntoa (*ptr));
+	      printf
+		("Trying to send an ICP packet of %u bytes to the internet address %s...\n",
+		 length, inet_ntoa (*ptr));
 	    }
 #endif
 	  else
@@ -494,6 +688,25 @@ main (argc, argv)
 		}
 #endif
 	    }
+	  if (!udp && !ssl)
+	    if ((files = fdopen (sockfd, "r")) == NULL)
+	      err_sys ("Cannot fdopen");
+#ifdef OPENSSL
+	  if (ssl)
+	    {
+	      SSL_set_fd (sslh, sockfd);
+	      if (SSL_connect (sslh) == -1)
+		if ((errno == EINTR) && (timeout_flag))
+		  {
+		    printf ("Timeout while starting SSL\n");
+		    continue;
+		  }
+	      if (verbose)
+		printf ("SSL connection using %s\n", SSL_get_cipher (sslh));
+	      /* We could check the server's certificate or other funny
+	         things */
+	    }
+#endif
 	}
       /* Not T/TCP */
       else
@@ -501,13 +714,15 @@ main (argc, argv)
 	  /* No initial connection */
 	}
       if ((port_to_use == USE_ECHO) || (port_to_use == USE_DISCARD) ||
-	  (port_to_use == USE_HTTP) || (port_to_use == USE_ICP))
+	  (port_to_use == USE_HTTP) || (port_to_use == USE_ICP) ||
+	  (port_to_use == USE_SMTP))
 	{
-#ifdef TTCP
+#ifdef USE_TTCP
 	  if (ttcp)
 	    {
 	      if (sendto (sockfd, sendline, n, MSG_EOF,
-		   (struct sockaddr *) &serv_addr, sizeof (serv_addr)) != n)
+			  (struct sockaddr *) &serv_addr,
+			  sizeof (serv_addr)) != n)
 		err_sys ("sendto error on socket");
 	      if (verbose)
 		{
@@ -518,16 +733,53 @@ main (argc, argv)
 #endif
 	  if (!udp)
 	    {
+	      if (!ssl)
+		{
+		  /* Write something to the server */
+		  if (writen (sockfd, sendline, n) != n)
+		    {
+		      if ((nr < 0 || nr != n) && timeout_flag)
+			{
+			  nr = n;
+			  printf ("Timeout while writing\n");
+			  continue;
+			}
+		      else
+			err_sys ("writen error on socket");
+		    }
+		}
+#ifdef OPENSSL
+	      else
+		{
+		  if ((rc = SSL_write (sslh, sendline, n)) != n)
+		    {
+		      if ((nr < 0 || nr != n) && timeout_flag)
+			{
+			  nr = n;
+			  printf ("Timeout while writing\n");
+			  continue;
+			}
+		      else
+			{
+			  sslcode = ERR_get_error ();
+			  err_sys ("SSL_write error on socket: %s",
+				   ERR_error_string (sslcode, NULL));
+			}
+		    }
+		}
+#endif
 	      /* Write something to the server */
 	      if (writen (sockfd, sendline, n) != n)
-		if ((nr < 0 || nr != n) && timeout_flag)
-		  {
-		    nr = n;
-		    printf ("Timeout while writing\n");
-		    continue;
-		  }
-		else
-		  err_sys ("writen error on socket");
+		{
+		  if ((nr < 0 || nr != n) && timeout_flag)
+		    {
+		      nr = n;
+		      printf ("Timeout while writing\n");
+		      continue;
+		    }
+		  else
+		    err_sys ("writen error on socket");
+		}
 	    }
 	  else
 	    {
@@ -566,19 +818,32 @@ main (argc, argv)
 	    }
 	}
       if ((port_to_use == USE_ECHO) || (port_to_use == USE_CHARGEN) ||
-	  (port_to_use == USE_HTTP) || (port_to_use == USE_ICP))
+	  (port_to_use == USE_HTTP) || (port_to_use == USE_ICP) ||
+	  (port_to_use == USE_SMTP))
 	{
 	  if (!udp)
 	    {
-	      if (!http)
+	      if (!http && !smtp)
 		{
 		  /* Read from the server */
-		  nr = readline (sockfd, recvline, n, stop_at_newlines);
+		  nr = readline (files, recvline, n, stop_at_newlines);
 		}
 #ifdef HTTP
-	      else
+	      else if (http)
 		{
-		  nr = read_from_server (sockfd);
+		  if (!ssl)
+		    channel.fs = files;
+#ifdef OPENSSL
+		  else
+		    channel.ssl = sslh;
+#endif
+		  nr = read_from_server (channel, ssl);
+		}
+#endif
+#ifdef SMTP
+	      else if (smtp)
+		{
+		  nr = smtp_read_response_from_server (files);
 		}
 #endif
 	    }
@@ -619,7 +884,7 @@ main (argc, argv)
 		   * sockets.
 		   */
 		  /*
-		   * BUG: in UDP, we should loop to read: we
+		   * Todo: in UDP, we should loop to read: we
 		   * can have several reads necessary.
 		   */
 		  alarm (0);
@@ -638,12 +903,13 @@ main (argc, argv)
 		}
 #endif
 	    }
-	  if (!http && !icp)
+	  if (!http && !icp && !smtp)
 	    {
 	      if ((nr < 0 || nr != n) && timeout_flag)
 		/* if ((nr < 0 || nr != n) && (errno == EINTR) && timeout_flag) */
 		{
-		  printf ("Timeout while reading (%d byte(s) read)\n", (nr == -1) ? 0 : nr);
+		  printf ("Timeout while reading (%d byte(s) read)\n",
+			  (nr == -1) ? 0 : nr);
 		  nr = n;
 #ifdef FLUSH_OUTPUT
 		  if (fflush ((FILE *) NULL) != 0)
@@ -654,14 +920,16 @@ main (argc, argv)
 		  continue;
 		}
 	      if (nr < 0 || nr != n)
-		err_sys ("readline error: %d bytes read, %d bytes requested", nr, n);
+		err_sys ("readline error: %d bytes read, %d bytes requested",
+			 nr, n);
 	    }
 	  else
 	    /* This is HTTP */
 	    {
 	      if ((nr < 0) && (errno == EINTR) && (timeout_flag))
 		{
-		  printf ("Timeout while reading (%d byte(s) read)\n", (nr == -1) ? 0 : nr);
+		  printf ("Timeout while reading (%d byte(s) read)\n",
+			  (nr == -1) ? 0 : nr);
 #ifdef FLUSH_OUTPUT
 		  if (fflush ((FILE *) NULL) != 0)
 		    {
@@ -670,11 +938,24 @@ main (argc, argv)
 #endif
 		  continue;
 		}
+	      if (nr < 0)
+		{
+		  err_ret ("Error reading HTTP reply");
+		}
 	    }
 	  if (verbose)
 	    printf ("%d bytes read from server.\n", nr);
 	}
       /* That's all, folks */
+      if (http)
+	{
+#ifdef OPENSSL
+	  if (ssl)
+	    SSL_shutdown (channel.ssl);
+	  else
+#endif
+	    fclose (channel.fs);
+	}
       close (sockfd);
 
       (void) gettimeofday (&newtv, (struct timezone *) NULL);
@@ -712,7 +993,7 @@ main (argc, argv)
 	      recvline[strlen (sendline)] = 0;
 	      if (strcmp (sendline, recvline) != 0)
 		{
-		  /* BUG: it does not work if the size is lower than the
+		  /* Todo: it does not work if the size is lower than the
 		     length of CHARGENERATED */
 		  printf (" I got back:\n%s\n", recvline);
 		  printf (" instead of the most common:\n%s\n", sendline);
@@ -740,7 +1021,18 @@ main (argc, argv)
 	}
       if (number > 1)
 	{
+#ifdef OPENSSL
+	  if (ssl)
+	    {
+	      /* SSL_clear (sslh); No, we have to free. Bug #130151 */
+	      SSL_free (sslh);
+	    }
+#endif
+#ifdef HAVE_USLEEP
+	  usleep (wait * 1000000);
+#else
 	  sleep (wait);
+#endif
 	}
     }
   printstats ();
@@ -748,6 +1040,7 @@ main (argc, argv)
     exit (0);
   else
     exit (1);
+  /* It would be nice to clean here (SSL, etc) */
 }
 
 
@@ -765,12 +1058,15 @@ printstats ()
 	printf ("Warning: %d message(s) lost (%d %%)\n", attempts - successes,
 		((attempts - successes) * 100) / attempts);
       printf ("Minimum time: %d.%06d seconds (%.0f bytes per sec.)\n",
-      (int) min.tv_sec, (int) min.tv_usec, (double) size / tv2double (min));
+	      (int) min.tv_sec, (int) min.tv_usec,
+	      (double) size / tv2double (min));
       printf ("Maximum time: %d.%06d seconds (%.0f bytes per sec.)\n",
-      (int) max.tv_sec, (int) max.tv_usec, (double) size / tv2double (max));
+	      (int) max.tv_sec, (int) max.tv_usec,
+	      (double) size / tv2double (max));
       tvavg (&total, successes);
       printf ("Average time: %d.%06d seconds (%.0f bytes per sec.)\n",
-	      (int) total.tv_sec, (int) total.tv_usec, (double) size / tv2double (total));
+	      (int) total.tv_sec, (int) total.tv_usec,
+	      (double) size / tv2double (total));
       /* The number of bytes/second, as printed above, is not really
          meaningful: size does not reflect the number of bytes exchanged.
          With echo, N = 2*size, with discard, N = size, with http, N = size + (response)... */
@@ -779,7 +1075,7 @@ printstats ()
 	  if (results[i].valid)
 	    good_results[j++] = results[i].timevalue;
 	}
-      if (successes != j)	/* Bug! */
+      if (successes != j)	/* Todo: bug! */
 	err_quit ("successes (%d) is different from j (%d)", successes, j);
       qsort (good_results, successes, sizeof (struct timeval), tvcmp);
       /*
@@ -806,7 +1102,8 @@ printstats ()
 	  tvavg (&median, 2);
 	}
       printf ("Median  time: %d.%06d seconds (%.0f bytes per sec.)\n",
-	      (int) median.tv_sec, (int) median.tv_usec, (double) size / tv2double (median));
+	      (int) median.tv_sec, (int) median.tv_usec,
+	      (double) size / tv2double (median));
     }
 }
 
